// src/constants.ts
var SocketDisconnectCode;
((SocketDisconnectCode2) => {
  SocketDisconnectCode2[SocketDisconnectCode2["CLOSE_NORMAL"] = 1000] = "CLOSE_NORMAL";
  SocketDisconnectCode2[SocketDisconnectCode2["CLOSE_GOING_AWAY"] = 1001] = "CLOSE_GOING_AWAY";
  SocketDisconnectCode2[SocketDisconnectCode2["CLOSE_PROTOCOL_ERROR"] = 1002] = "CLOSE_PROTOCOL_ERROR";
  SocketDisconnectCode2[SocketDisconnectCode2["CLOSE_UNSUPPORTED"] = 1003] = "CLOSE_UNSUPPORTED";
  SocketDisconnectCode2[SocketDisconnectCode2["CLOSE_NO_STATUS"] = 1005] = "CLOSE_NO_STATUS";
  SocketDisconnectCode2[SocketDisconnectCode2["CLOSE_ABNORMAL"] = 1006] = "CLOSE_ABNORMAL";
  SocketDisconnectCode2[SocketDisconnectCode2["CLOSE_TOO_LARGE"] = 1009] = "CLOSE_TOO_LARGE";
  SocketDisconnectCode2[SocketDisconnectCode2["CLOSE_EXTENSION_REQUIRED"] = 1010] = "CLOSE_EXTENSION_REQUIRED";
  SocketDisconnectCode2[SocketDisconnectCode2["CLOSE_INTERNAL_ERROR"] = 1011] = "CLOSE_INTERNAL_ERROR";
  SocketDisconnectCode2[SocketDisconnectCode2["CLOSE_SERVICE_RESTART"] = 1012] = "CLOSE_SERVICE_RESTART";
  SocketDisconnectCode2[SocketDisconnectCode2["CLOSE_TRY_AGAIN_LATER"] = 1013] = "CLOSE_TRY_AGAIN_LATER";
  SocketDisconnectCode2[SocketDisconnectCode2["CLOSE_TLS_HANDSHAKE"] = 1015] = "CLOSE_TLS_HANDSHAKE";
})(SocketDisconnectCode ||= {});
var NON_RECONNECT_CODES = [
  1000 /* CLOSE_NORMAL */,
  1001 /* CLOSE_GOING_AWAY */,
  1003 /* CLOSE_UNSUPPORTED */,
  1009 /* CLOSE_TOO_LARGE */,
  1010 /* CLOSE_EXTENSION_REQUIRED */,
  1013 /* CLOSE_TRY_AGAIN_LATER */,
  1015 /* CLOSE_TLS_HANDSHAKE */
];
var ManagedSocketErrorType;
((ManagedSocketErrorType2) => {
  ManagedSocketErrorType2["INVALID_MESSAGE_SHAPE"] = "Invalid message shape.";
  ManagedSocketErrorType2["CATASTROPHIC_CLOSE"] = "Catastrophic close code";
  ManagedSocketErrorType2["SOCKET_ERROR"] = "WebSocket error";
  ManagedSocketErrorType2["CONNECTION_REJECTED"] = "Connection rejected";
})(ManagedSocketErrorType ||= {});

// src/errors.ts
class ManagedSocketError extends Error {
  originalError;
  type = "WebSocket error" /* SOCKET_ERROR */;
  constructor(message, originalError, type) {
    super(message);
    this.name = "ManagedSocketError";
    this.originalError = originalError;
    this.type = type ?? this.type;
  }
}

// src/types.ts
var MESSAGE_TYPES = ["heartbeat", "message"];
function parseMessageType(value) {
  if (typeof value !== "string" || !MESSAGE_TYPES.includes(value)) {
    throw new ManagedSocketError("Failed to parse incoming message shape", new Error("Invalid message type"), "Invalid message shape." /* INVALID_MESSAGE_SHAPE */);
  }
  return value;
}
function parseAnyScoketMessage(value) {
  if (typeof value !== "object" || value === null) {
    throw new ManagedSocketError("Failed to parse incoming message shape: value was not an object", new Error("Invalid message shape"), "Invalid message shape." /* INVALID_MESSAGE_SHAPE */);
  }
  if (!("type" in value) || !("data" in value)) {
    throw new ManagedSocketError("Failed to parse incoming message shape: missing type or data", new Error("Invalid message shape"), "Invalid message shape." /* INVALID_MESSAGE_SHAPE */);
  }
  const { type: incomingType, data: incomingData } = value;
  const type = parseMessageType(incomingType);
  if (type === "heartbeat") {
    if (typeof incomingData !== "string") {
      throw new ManagedSocketError("Failed to parse incoming message shape: heartbeat data was not a string", new Error("Invalid message shape"), "Invalid message shape." /* INVALID_MESSAGE_SHAPE */);
    }
    return {
      type,
      data: incomingData
    };
  }
  if (type === "message") {
    return {
      type,
      data: incomingData
    };
  }
  throw new ManagedSocketError("Failed to parse incoming message shape", new Error("Invalid message type"), "Invalid message shape." /* INVALID_MESSAGE_SHAPE */);
}

// src/messages.ts
function createDataMessage(data) {
  return {
    type: "message",
    data
  };
}
function createHeartbeatMessage() {
  return {
    type: "heartbeat",
    data: new Date().toISOString()
  };
}
function serialize(message) {
  return JSON.stringify(message);
}
function deserialize(data) {
  try {
    const asObj = JSON.parse(data);
    return parseAnyScoketMessage(asObj);
  } catch (error) {
    throw new ManagedSocketError("Failed to parse incoming message shape", error instanceof Error ? error : new Error("Unknown error"), "Invalid message shape." /* INVALID_MESSAGE_SHAPE */);
  }
}
function prepare(data) {
  return serialize(createDataMessage(data));
}
function isCatastrophicCloseCode(code) {
  return NON_RECONNECT_CODES.includes(code);
}
var SocketSpeak = {
  createDataMessage,
  createHeartbeatMessage,
  serialize,
  deserialize,
  isCatastrophicCloseCode,
  prepare
};

// src/managed-socket.ts
var ONE_SECOND = 1000;
var HEARTBEAT_INTERVAL = 30 * ONE_SECOND;
var MAX_RECONNECT_DELAY = 30 * ONE_SECOND;

class ManagedSocket {
  url;
  state = "connecting";
  ws;
  onMessageHandler = null;
  onErrorHandler = null;
  onOpenHandler = null;
  onCloseHandler = null;
  onCatastrophicErrorHandler = null;
  reconnectAttempts;
  maxReconnectDelay;
  heartbeatInterval;
  messageQueue = [];
  attemptReconnect = true;
  debugMode = false;
  constructor(url) {
    this.url = url;
    this.ws = this.connect();
    this.reconnectAttempts = 0;
    this.maxReconnectDelay = MAX_RECONNECT_DELAY;
    this.heartbeatInterval = null;
  }
  get connected() {
    return this.ws.readyState === WebSocket.OPEN;
  }
  get native() {
    return this.ws;
  }
  set debug(value) {
    this.debugMode = value;
  }
  debugMessage(...args) {
    if (this.debugMode) {
      const output = ["[ManagedSocketDebug]", ...args];
      console.log(...output);
    }
  }
  sendQueuedMessages() {
    const messagesToSend = [...this.messageQueue];
    this.messageQueue = [];
    for (const message of messagesToSend) {
      this.send(message);
    }
  }
  connect() {
    this.ws = new WebSocket(this.url);
    this.ws.addEventListener("open", () => {
      this.reconnectAttempts = 0;
      if (this.state === "reconnecting") {
        this.debugMessage("Reconnected to socket server");
      }
      this.onOpenHandler?.();
      this.state = "open";
      this.startHeartbeat();
      this.sendQueuedMessages();
    });
    this.ws.addEventListener("close", (event) => {
      const { code } = event;
      this.debugMessage(code);
      this.onCloseHandler?.();
      if (SocketSpeak.isCatastrophicCloseCode(code)) {
        this.handleCatastrophicError(code);
      }
      this.state = "closed";
      if (this.attemptReconnect) {
        this.debugMessage("Socket connection closed. Attempting reconnection...");
        this.stopHeartbeat();
        this.reconnect();
      }
    });
    this.ws.addEventListener("error", (error) => {
      this.debugMessage("WebSocket error", error);
      const errorType = this.state === "connecting" ? "Connection rejected" /* CONNECTION_REJECTED */ : "WebSocket error" /* SOCKET_ERROR */;
      const socketError = new ManagedSocketError(errorType, new Error(error.currentTarget?.toString()), errorType);
      if (this.onErrorHandler !== null) {
        return this.onErrorHandler(socketError);
      }
      throw error;
    });
    this.ws.addEventListener("message", (event) => {
      const message = SocketSpeak.deserialize(event.data.toString());
      if (message.type === "heartbeat") {
        this.debugMessage("Received heartbeat");
        return;
      }
      if (message.data === undefined || message.data === null) {
        this.debugMessage("Received message with no data");
        return;
      }
      this.debugMessage("Received message", message.data);
      this.onMessageHandler?.(message.data);
    });
    return this.ws;
  }
  handleCatastrophicError(code) {
    this.state = "terminated";
    this.attemptReconnect = false;
    this.debugMessage("Socket connection terminated due to non-reconnect close code");
    const socketError = new ManagedSocketError("Socket connection terminated due to non-reconnect close code", new Error(code.toString()), "Catastrophic close code" /* CATASTROPHIC_CLOSE */);
    if (this.onCatastrophicErrorHandler !== null) {
      return this.onCatastrophicErrorHandler(socketError);
    }
    throw socketError;
  }
  onMessage(handler) {
    this.onMessageHandler = handler;
  }
  onError(handler) {
    this.onErrorHandler = handler;
  }
  onOpen(handler) {
    this.onOpenHandler = handler;
  }
  onClose(handler) {
    this.onCloseHandler = handler;
  }
  onCatastrophicError(handler) {
    this.onCatastrophicErrorHandler = handler;
  }
  reconnect() {
    this.attemptReconnect = true;
    this.state = "reconnecting";
    const delay = Math.min(1000 * 2 ** this.reconnectAttempts, this.maxReconnectDelay);
    this.debugMessage(`Attempting to reconnect in ${delay}ms`);
    setTimeout(() => {
      this.reconnectAttempts++;
      this.connect();
    }, delay);
  }
  close() {
    this.attemptReconnect = false;
    this.state = "closing";
    this.ws.close();
  }
  startHeartbeat() {
    this.heartbeatInterval = setInterval(() => {
      if (this.ws.readyState === WebSocket.OPEN) {
        this.ws.send(SocketSpeak.serialize(SocketSpeak.createHeartbeatMessage()));
      }
    }, HEARTBEAT_INTERVAL);
  }
  stopHeartbeat() {
    if (this.heartbeatInterval !== null) {
      clearInterval(this.heartbeatInterval);
      this.heartbeatInterval = null;
    }
  }
  send(data) {
    if (this.ws.readyState !== WebSocket.OPEN) {
      this.debugMessage("WebSocket is not open. Queuing message.");
      this.messageQueue.push(data);
      return;
    }
    this.ws.send(SocketSpeak.prepare(data));
  }
}
export {
  SocketSpeak,
  SocketDisconnectCode,
  ManagedSocketErrorType,
  ManagedSocketError,
  ManagedSocket
};
